// CopySession.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "CopySession.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCopySession dialog


CCopySession::CCopySession(CWnd* pParent /*=NULL*/)
	: CDialog(CCopySession::IDD, pParent)
{
  m_copy_all_tubes = false;
  m_curr = 1;
	//{{AFX_DATA_INIT(CCopySession)
	m_copy_to = 1;
	m_copy_from = 2;
	//}}AFX_DATA_INIT
}

/**
* Sets the default destination value in the copy session dialog box
* @return unsigned int the user chose within the copy session dialog box for the source session.
*/
unsigned int CCopySession::GetSource() 
{
  return m_copy_from;
}


/**
* Sets the default destination value in the copy session dialog box
* @return unsigned int the user chose within the copy session dialog box for the destination session.
*/
unsigned int CCopySession::GetDestination()
{
  return m_copy_to;
}


/**
* Sets the default destination value in the copy session dialog box
* @param source unsigned int which sets the default display source session value in the dialog box
* @return unsigned int the user chose within the copy session dialog box.
*/
bool CCopySession::SetSource(unsigned int source)
{
  if(source == 0) 
  {
    return false;
  }

  m_copy_from = source;

  return true;

}

/**
* Sets the default destination value in the copy session dialog box
* @param destination unsigned int representing the destination session number, must be >= 1
* @return bool indicating success(true) or failure(false)
*/
bool CCopySession::SetDestination(unsigned int destination)
{
  if(destination == 0) 
  {
    return false;
  }

  m_copy_to = destination;

  return true;
}

/**
* Sets the current session number in the copy session dialog box
* @param curr unsigned int representing the current session number, must be >= 1
* @return bool indicating success(true) or failure(false)
*/
bool CCopySession::SetCurrent(unsigned int curr)
{
  if(curr == 0) 
  {
    return false;
  }

  m_curr = curr;

  return true;
}

void CCopySession::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCopySession)
	DDX_Text(pDX, IDC_EDIT_COPYTO, m_copy_to);
	DDX_Text(pDX, IDC_EDIT_COPYFROM, m_copy_from);
	//}}AFX_DATA_MAP
}

/**
* Sets the default destination value in the copy session dialog box
* @return bool indicating whether the user checked the copy all roots in the tube checkbox
*/
bool CCopySession::GetCopyAll() 
{
  return m_copy_all_tubes;
//  return m_copy_all ? true : false;
}


BEGIN_MESSAGE_MAP(CCopySession, CDialog)
	//{{AFX_MSG_MAP(CCopySession)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCopySession message handlers

BOOL CCopySession::OnInitDialog() 
{
	CDialog::OnInitDialog();

  unsigned default_radio = m_copy_all_tubes ? IDC_RADIO_COPY_ALLWINDOWTUBE : IDC_RADIO_COPY_CURRENTWINDOWTUBE;
  CheckRadioButton(IDC_RADIO_COPY_CURRENTWINDOWTUBE, IDC_RADIO_COPY_ALLWINDOWTUBE, default_radio);	

  CWnd* wnd = GetDlgItem(IDC_STATIC_COPYSESSION_CURSES);
  if (wnd)  
  {
    CString str;
    str.Format("(Current session is %d.)", m_curr);
    wnd->SetWindowText(str);
  }

  return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CCopySession::OnOK() 
{
  unsigned int ret = GetCheckedRadioButton(IDC_RADIO_COPY_CURRENTWINDOWTUBE, IDC_RADIO_COPY_ALLWINDOWTUBE);
  m_copy_all_tubes = (ret == IDC_RADIO_COPY_ALLWINDOWTUBE);
	
	CDialog::OnOK();
}
