// ModeHandler.h: classes for handling different modes of the application.
//
//////////////////////////////////////////////////////////////////////

#ifndef AFX_ROOTFLY_MODEHANDLER__INCLUDED_
#define AFX_ROOTFLY_MODEHANDLER__INCLUDED_

#include <afxwin.h>
//#include "DataStructures.h"
//#include "ImageToWindowConverter.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CRootflyDlg;

// Abstract base class for handling different modes in the Rootfly application.
class ModeHandler
{
public:
  ModeHandler(CRootflyDlg* dlg) { m_dlg = dlg; }
  virtual void OnMouseMove(UINT nFlags, CPoint point) = 0;
  virtual void OnLButtonDown(UINT nFlags, CPoint point) = 0;
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point) = 0;
  virtual void OnRButtonDown(UINT nFlags, CPoint point) = 0;
  virtual CString GetStatusText() = 0;
protected:
  CRootflyDlg* m_dlg;
};

// Group 1
class ModeHandlerMODE_STARTINGMOVEALL : public ModeHandler
{
public:
  ModeHandlerMODE_STARTINGMOVEALL(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 1
class ModeHandlerMODE_MOVINGSINGLEROOT : public ModeHandler
{
public:
  ModeHandlerMODE_MOVINGSINGLEROOT(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 2
class ModeHandlerMODE_NOFILELOADED : public ModeHandler
{
public:
  ModeHandlerMODE_NOFILELOADED(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 2
class ModeHandlerMODE_MOVINGALLROOTS : public ModeHandler
{
public:
  ModeHandlerMODE_MOVINGALLROOTS(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 3
class ModeHandlerMODE_SIZINGDIAMETER : public ModeHandler
{
public:
  ModeHandlerMODE_SIZINGDIAMETER(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 3
class ModeHandlerMODE_DRAWINGNEWROOT : public ModeHandler
{
public:
  ModeHandlerMODE_DRAWINGNEWROOT(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDown(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point) ;
  virtual void OnRButtonDown(UINT nFlags, CPoint point) ;
  virtual CString GetStatusText() ;
};

// Group 4
class ModeHandlerMODE_ROOTSELECTED : public ModeHandler
{
public:
  ModeHandlerMODE_ROOTSELECTED(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 4
class ModeHandlerMODE_NOIMAGEDISPLAYED : public ModeHandler
{
public:
  ModeHandlerMODE_NOIMAGEDISPLAYED(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 5
class ModeHandlerMODE_EXTENDINGENDPOINT : public ModeHandler
{
public:
  ModeHandlerMODE_EXTENDINGENDPOINT(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDown(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point) ;
  virtual void OnRButtonDown(UINT nFlags, CPoint point) ;
  virtual CString GetStatusText() ;
};

// Group 5
class ModeHandlerMODE_IMAGEDISPLAYED : public ModeHandler
{
public:
  ModeHandlerMODE_IMAGEDISPLAYED(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDown(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point) ;
  virtual void OnRButtonDown(UINT nFlags, CPoint point) ;
  virtual CString GetStatusText() ;
};

// Group 6
class ModeHandlerMODE_RESIZINGDIAMETER : public ModeHandler
{
public:
  ModeHandlerMODE_RESIZINGDIAMETER(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 6
class ModeHandlerMODE_MOVINGRPOINT : public ModeHandler
{
public:
  ModeHandlerMODE_MOVINGRPOINT(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 6
class ModeHandlerMODE_ADDPOINT: public ModeHandler
{
public:
  ModeHandlerMODE_ADDPOINT(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 7
class ModeHandlerMODE_LOCATINGDIAMETER : public ModeHandler
{
public:
  ModeHandlerMODE_LOCATINGDIAMETER(CRootflyDlg* dlg) : ModeHandler(dlg) {}
  virtual void OnMouseMove(UINT nFlags, CPoint point);
  virtual void OnLButtonDown(UINT nFlags, CPoint point);
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

// Group 7
class ModeHandlerMODE_REPOSITIONDIAMETER : public ModeHandler
{
public:
  ModeHandlerMODE_REPOSITIONDIAMETER(CRootflyDlg* dlg) : ModeHandler(dlg){}
  virtual void OnMouseMove(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDown(UINT nFlags, CPoint point) ;
  virtual void OnLButtonDblClk(UINT nFlags, CPoint point);
  virtual void OnRButtonDown(UINT nFlags, CPoint point);
  virtual CString GetStatusText();
};

#endif // !defined(AFX_ROOTFLY_MODEHANDLER__INCLUDED_)
