// OptionsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "OptionsDlg.h"
#include "ImageFileFormat.h"
#include "FormatDateDlg.h"
#include <vector>
#include <string>
#include <sstream>
using namespace std;

#pragma warning(disable:4355)  // Remove compiler warning (this used in initializer list)

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Sample image filename formats (added by STB):
//   -exp_T[%3tube]_L[%3window]_[%4year].[%2month].[%2day]_[%6timestamp]_[%3session]_[%3initials].jpg (Bartz)
//    3 numbers for tube
//    3 numbers for window
//    4 numbers for year
//    2 numbers for month
//    2 numbers for day
//    6 numbers for timestamp
//    3 numbers for session
//    3 letters for operator initials
//      Example:  
//        bartz_T001_L009_2006.02.21_073856_001_KLF.jpg
//
//   - exp_TubeTTT_SS_WW.jpg    (from Eric)
//     2 numbers for tube, 1 for transect
//     2 numbers for session
//     2 numbers for window
//
//   - exp_SS_TTT_WW.jpg        (from Cory)
//     2 numbers for session
//     2 numbers for tube, 1 for transect
//     2 numbers for window
//
//   - exp_TTT_SS_WW.jpg
//     2 numbers for tube, 1 for transect
//     2 numbers for session
//     2 numbers for window
//
//   - a#TT#WWW#S[S].jpg        (from Alex)
//     2 numbers for tube
//     3 numbers for window
//     1 or 2 letters for session (e.g., 'a', 'b', ..., 'z', 'aa', 'ab', ...)
//
//   - Tanner_17_31b_01
//
//   - psret_plot2_03_01_13_04
//     [expt]_plot[tube]_[window]_DD_MM_YY
//
//   - Musser_Tube10a_10_25.jpg

/////////////////////////////////////////////////////////////////////////////
// OptionsDlg dialog


OptionsDlg::OptionsDlg(ImageFileFormat* image_file_format, CWnd* pParent /*=NULL*/)
	: CDialog(OptionsDlg::IDD, pParent),
    iff(*image_file_format),
    m_lbIncluded(image_file_format, this),
    m_lbNotIncluded()
    //m_lbNotIncluded(image_file_format)
{
	//{{AFX_DATA_INIT(OptionsDlg)
	//}}AFX_DATA_INIT
}


void OptionsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(OptionsDlg)
	DDX_Control(pDX, IDC_OPTIONS_DATE_FORMAT, m_bDate);
	DDX_Control(pDX, IDC_OPTIONS_LIST_NOTINCLUDED, m_lbNotIncluded);
	DDX_Control(pDX, IDC_OPTIONS_LIST_INCLUDED, m_lbIncluded);
	DDX_Control(pDX, IDC_OPTIONS_STATIC_EX, m_sEx);
	DDX_Control(pDX, IDC_OPTIONS_EDIT_PREFIX, m_ePrefix);
	DDX_Control(pDX, IDC_OPTIONS_EDIT_EXT, m_eExt);
	DDX_Control(pDX, IDC_OPTIONS_EDIT_DELIM, m_eDelim);
	DDX_Control(pDX, IDC_OPTIONS_REMOVE, m_bRemove);
	DDX_Control(pDX, IDC_OPTIONS_ADD, m_bAdd);
	DDX_Control(pDX, IDC_OPTIONS_BARTZ, m_bBartz);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(OptionsDlg, CDialog)
	//{{AFX_MSG_MAP(OptionsDlg)
	ON_LBN_SELCHANGE(IDC_OPTIONS_LIST_INCLUDED, OnSelchangeOptionsListIncluded)
	ON_BN_CLICKED(IDC_OPTIONS_BARTZ, OnOptionsBartz)
	ON_BN_CLICKED(IDC_OPTIONS_ADD, OnOptionsAdd)
	ON_EN_KILLFOCUS(IDC_OPTIONS_EDIT_DELIM, OnKillfocusOptionsEditDelim)
	ON_EN_KILLFOCUS(IDC_OPTIONS_EDIT_EXT, OnKillfocusOptionsEditExt)
	ON_EN_KILLFOCUS(IDC_OPTIONS_EDIT_PREFIX, OnKillfocusOptionsEditPrefix)
	ON_LBN_SELCHANGE(IDC_OPTIONS_LIST_NOTINCLUDED, OnSelchangeOptionsListNotincluded)
	ON_BN_CLICKED(IDC_OPTIONS_REMOVE, OnOptionsRemove)
	ON_BN_CLICKED(IDC_OPTIONS_STATIC_IFF_BOX, OnOptionsStaticIffBox)
	ON_BN_CLICKED(IDOK_OPTIONS, OnOptions)
	ON_BN_CLICKED(IDCANCEL_OPTIONS, OnOptionsCancel)
	ON_LBN_KILLFOCUS(IDC_OPTIONS_LIST_INCLUDED, OnKillfocusOptionsListIncluded)
	ON_LBN_KILLFOCUS(IDC_OPTIONS_LIST_NOTINCLUDED, OnKillfocusOptionsListNotincluded)
	ON_BN_CLICKED(IDC_OPTIONS_MOVE_UP, OnOptionsMoveUp)
	ON_BN_CLICKED(IDC_OPTIONS_MOVE_DOWN, OnOptionsMoveDown)
	ON_BN_CLICKED(IDC_OPTIONS_DATE_FORMAT, OnOptionsDateFormat)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// OptionsDlg message handlers

BOOL OptionsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  // Delimiter Text
  
  CString delim = iffi.delim;
  m_eDelim.ReplaceSel(delim);
  m_eDelim.EnableWindow();

  // Extension Text
  m_eExt.ReplaceSel(iffi.filetype);
  m_eExt.EnableWindow();
  
  // ListBox
  m_Order = iffi.InfoOrder;
  for(int i = 0; i < m_Order.size(); i++)
    if(m_Order[i].included)
      m_lbIncluded.AddString(m_Order[i].GetName());
    else
      m_lbNotIncluded.AddString(m_Order[i].GetName());
  m_lbIncluded.EnableWindow();
  m_lbNotIncluded.EnableWindow();

  // Buttons
  m_bAdd.EnableWindow(FALSE);
  m_bRemove.EnableWindow(FALSE);
  m_bDate.ShowWindow(SW_HIDE);

  // Prefix Text
  m_ePrefix.EnableWindow(FALSE);

  // Example Text  
  CString ex = GetImageFileFormat();
  m_sEx.SetWindowText(ex);

  UpdateData();

  //initialize date format
/*  iffi.m_date_format.date_delim = "/";
  iffi.m_date_format.four_year = true;
  iffi.m_date_format.order = {MONTH,DAY,YEAR}; */

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

CString OptionsDlg::GetImageFileFormat()
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  CString str = "";
  CString delim, filetype, tmp;
  m_eDelim.GetWindowText(delim);
  m_eExt.GetWindowText(filetype);

  for(int i = 0; i < NUM_OF_ELEMENTS; i++)
  {
    if(m_Order[i].included)
    {
      str += m_Order[i].prefix;
      switch(m_Order[i].element)
      {
      case EXPT: tmp = "[expt]"; break;
      case TUBE: tmp = "[tube]"; break;
      case WINDOW: tmp = "[win]"; break;
      case SESSION: tmp = "[sess]"; break;
      case SDATE: tmp = GetDateFormat(); break;
      case TIME: tmp = "[time]"; break;
      case INITIALS: tmp = "[user]"; break;
      case UNUSED1: tmp = "[unused]"; break;
      case UNUSED2: tmp = "[unused]"; break;
      case UNUSED3: tmp = "[unused]"; break;
      case UNUSED4: tmp = "[unused]"; break;
      }
      str += tmp;
      str += delim;
    }
  }
  if(str.GetLength())
  {
    str.SetAt(str.GetLength()-1,'.');
    str += filetype;
  }
  
  return str;
}

CString OptionsDlg::GetDateFormat()
{
  CString temp;
  Date_Format format;
  format = iff.GetImageFileFormatInfo().m_date_format;
  temp = "[";
  for(int i = 0; i < format.order.size(); i++)
  {
    if(format.order.at(i) == MONTH)
    {
      temp += "mm";
    } else if(format.order.at(i) == DAY)
    {
      temp += "dd";
    } else if(format.order.at(i) == YEAR)
    {
      if(format.four_year) temp += "yyyy";
      else temp+= "yy";
    }
    if(i != format.order.size()-1) temp += format.date_delim;
  }
  temp += "]";
  return temp;
}

void OptionsDlg::SetFormat()
{
  ImageFileFormatInfo temp = iff.GetImageFileFormatInfo();
  CString filetype, delim;
  m_eExt.GetWindowText(filetype);
  m_eDelim.GetWindowText(delim);

  ImageFileFormatInfo iffi;
  iffi.InfoOrder = m_Order;
  iffi.m_date_format = temp.m_date_format;

  int sum = 0;
  for(int i = 0; i < m_Order.size(); i++)
    if(m_Order[i].included) sum++;
  iffi.numElementsInFileName = sum;

  iffi.filetype = filetype;
  iffi.delim = delim[0];

  iff.SetImageFileFormatInfo(iffi);
  
  // set example text at bottom of dialog
  CString ex = GetImageFileFormat();
  m_sEx.SetWindowText(ex);
}

void OptionsDlg::OnOptionsBartz() 
{
	int delim_len = m_eDelim.LineLength();
  int ext_len = m_eExt.LineLength();
  int ex_len = m_sEx.GetWindowTextLength();
  int prefix_len = m_ePrefix.LineLength();

	// Bartz format
  iff.SetToBartz();
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  m_eDelim.SetSel(0,delim_len);
  m_eDelim.ReplaceSel(CString(iffi.delim));

  m_eExt.SetSel(0,ext_len);
  m_eExt.ReplaceSel(iffi.filetype);

  m_ePrefix.SetSel(0,prefix_len);
  m_ePrefix.ReplaceSel("");

  int i = 0;
  m_lbIncluded.ResetContent();
  for(i = 0; i < 7; i++)
    m_lbIncluded.AddString(m_Order[i].GetName());

  m_lbNotIncluded.ResetContent();
  for(i = 7; i < NUM_OF_ELEMENTS; i++)
    m_lbNotIncluded.AddString(m_Order[i].GetName());
  
  SetFormat();

  m_bRemove.EnableWindow(false);
  m_bAdd.EnableWindow(false);
  m_ePrefix.EnableWindow(false);
}

void OptionsDlg::OnKillfocusOptionsEditDelim() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

	// Check for valid data
	CString tmp;
	m_eDelim.GetWindowText(tmp);
	if(tmp.GetLength() == 1)
	    SetFormat();
	else // invalid data
	{
	  m_eDelim.SetSel(0,m_eDelim.LineLength());
		CString d = iff.GetImageFileFormatInfo().delim;
		m_eDelim.ReplaceSel(d);
	}
}

void OptionsDlg::OnKillfocusOptionsEditExt() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

	// Check for valid data
  CString tmp;
  m_eExt.GetWindowText(tmp);
  if(tmp == "jpg" || tmp == "jpeg" || tmp == "bmp")
	  SetFormat();	
  else // invalid data -- unsupported image type
  {
	  m_eExt.SetSel(0,m_eExt.LineLength());
	  CString e = iff.GetImageFileFormatInfo().filetype;
	  m_eExt.ReplaceSel(e);
  }
}

void OptionsDlg::OnKillfocusOptionsEditPrefix() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

	int s = m_lbIncluded.GetCurSel();
  if(s == LB_ERR) return;

  // update prefix data for selected element
  CString tmp;
  m_ePrefix.GetWindowText(tmp);
  m_Order[s].prefix = tmp;

  SetFormat();
}

void OptionsDlg::OnSelchangeOptionsListIncluded() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  int s = m_lbIncluded.GetCurSel();
  if(s == LB_ERR) return;

  m_ePrefix.SetSel(0,m_ePrefix.LineLength());
  m_ePrefix.ReplaceSel(m_Order[s].prefix);
  m_ePrefix.EnableWindow();

  if(m_lbIncluded.GetCount() > 0 && m_lbIncluded.GetCurSel() != LB_ERR)
    m_bRemove.EnableWindow();

  if (m_Order[s].element == SDATE)
  {
    m_bDate.ShowWindow(SW_SHOW);
  }
  else
  {
    m_bDate.ShowWindow(SW_HIDE);
  }

  SetFormat();

  m_ePrefix.EnableWindow();
}

void OptionsDlg::OnSelchangeOptionsListNotincluded() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  int s = m_lbNotIncluded.GetCurSel() + m_lbIncluded.GetCount();

  // update prefix edit box
  m_ePrefix.SetSel(0,m_ePrefix.LineLength());
  m_ePrefix.ReplaceSel(m_Order[s].prefix);
  m_ePrefix.EnableWindow(false);

  if(m_lbNotIncluded.GetCount() > 0)
	  m_bAdd.EnableWindow();

  SetFormat();
}

void OptionsDlg::OnOptionsAdd() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  if(m_lbNotIncluded.GetCount() > 0)
  {
    int s;  // selection in not included list
    if((s = m_lbNotIncluded.GetCurSel()) == LB_ERR) return;
    int t = m_lbIncluded.GetCount();     // place where string is to be put
    
    CString str;
    m_lbNotIncluded.GetText(s,str);      // get the string
    m_lbNotIncluded.DeleteString(s);     // delete it from not included list

    m_lbIncluded.InsertString(t,str);    // insert string into included list

    // manipulate m_Order

    s += t+1;                              // this is where element should be in m_Order

//    {
//      CString tmp = "";
//      tmp.Format("s: %d\nt: %d",s,t);
//      AfxMessageBox(tmp);
//    }

    m_Order.insert(m_Order.begin() + t,1,m_Order[s]);  // insert element before t
    m_Order.erase(m_Order.begin() + s);                // erase old instance of element

    // set included flag
    m_Order[t].included = true;
    
    SetFormat();
    
    if(m_lbNotIncluded.GetCount() == 0)
      m_bAdd.EnableWindow(false);
    m_bRemove.EnableWindow(false);
  }
}

void OptionsDlg::OnOptionsRemove() 
{
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
  m_Order = iffi.InfoOrder;

  if(m_lbIncluded.GetCount() > 0)
  {
    int s;  // selection in included list
    if((s = m_lbIncluded.GetCurSel()) == LB_ERR) return;

    int t = m_lbNotIncluded.GetCount();  // place where string is to be put
    
    CString str;
    m_lbIncluded.GetText(s,str);         // get the string
    m_lbIncluded.DeleteString(s);        // delete the string

    m_lbNotIncluded.InsertString(t,str); // insert the string in its new place

    // manipulate m_Order
    m_Order.push_back(m_Order[s]);       // add copy of element to end of list
    m_Order.erase(m_Order.begin()+s);    // remove old element from list
    
    m_Order[NUM_OF_ELEMENTS-1].included = false;
          
    SetFormat();
    
    if(m_lbIncluded.GetCount() == 0)
      m_bRemove.EnableWindow(false);
    m_bAdd.EnableWindow(false);
    m_ePrefix.EnableWindow(false);
  }

  m_bDate.ShowWindow(SW_HIDE);
}

void OptionsDlg::OnOptionsStaticIffBox() 
{
  SetFormat();
}

void OptionsDlg::OnOptions() 
{
	ImageFileFormatInfo iffi, temp;
  CString delim,filetype;
  m_eDelim.GetWindowText(delim);
  m_eExt.GetWindowText(filetype);

  temp = iff.GetImageFileFormatInfo();
  // fill image file format info struct
  iffi.delim = delim[0];
  iffi.filetype = filetype;
  int sum = 0;
  for(int i = 0; i < m_Order.size(); i++)
    if(m_Order[i].included) sum++;
  iffi.numElementsInFileName = sum;

  iffi.InfoOrder = m_Order;

  // set image file format info
  iffi.m_date_format = temp.m_date_format;
  iff.SetImageFileFormatInfo(iffi);

	CDialog::OnOK();
}

void OptionsDlg::OnOptionsCancel() 
{
  CDialog::OnCancel();	
}

void OptionsDlg::OnOK()
{
  // Do nothing.  This prevents the dialog from closing when the user hits the enter key
}

void OptionsDlg::OnKillfocusOptionsListIncluded() 
{
	SetFormat();
}

void OptionsDlg::OnKillfocusOptionsListNotincluded() 
{
	SetFormat();
}

void OptionsDlg::OnOptionsMoveUp() 
{
  // return if no elements in list
	if(m_lbIncluded.GetCount() == 0) return;

  int s;

  // return if no element selected
  if((s = m_lbIncluded.GetCurSel()) == LB_ERR) return;

  // return if first element
  if(s == 0) return;

  // swap strings
  CString str;
  m_lbIncluded.GetText(s,str);
  m_lbIncluded.DeleteString(s);
  m_lbIncluded.InsertString(s-1,str);

  // swap order elements
  OrderElement tmp = m_Order[s];
  m_Order[s] = m_Order[s-1];
  m_Order[s-1] = tmp;

  SetFormat();

  m_ePrefix.EnableWindow(false);

  m_lbIncluded.SetCurSel(s-1);
}

void OptionsDlg::OnOptionsMoveDown() 
{
  // return if no elements exist
  if(m_lbIncluded.GetCount() == 0) return;

  int s;

  // return if no selection made
  if((s = m_lbIncluded.GetCurSel()) == LB_ERR) return;
  
  // return if last element
  if(s == m_lbIncluded.GetCount()-1) return;

  // swap strings
  CString str;
  m_lbIncluded.GetText(s,str);
  m_lbIncluded.DeleteString(s);
  m_lbIncluded.InsertString(s+1,str);

  // swap elements
  OrderElement tmp = m_Order[s];
  m_Order[s] = m_Order[s+1];
  m_Order[s+1] = tmp;

  SetFormat();

  m_ePrefix.EnableWindow(false);

  m_lbIncluded.SetCurSel(s+1);
}

// commented out this function since we reverted back to a CListBox. -- STB 6/11/07
void OptionsDlgDragListBox::Dropped (int nSrcIndex, CPoint pt)
{
//  DrawInsert(-1);
//  int nDestIndex = ItemFromPt(pt);
//
//  // check for movement; return if no movement
//  if (nSrcIndex == -1 || nDestIndex == -1)
//	return;
//	if (nDestIndex == nSrcIndex || nDestIndex == nSrcIndex+1)
//	return;
//
//  // moved - so let's swap around m_order
//  ImageFileFormatInfo iffi = m_iff.GetImageFileFormatInfo();
//  vector<OrderElement> order = iffi.InfoOrder;
//
//  // used with order vector - shorter names
//  int dest = nDestIndex;
//  int src = nSrcIndex;
//  
//  // insert string into listbox
//  InsertString(nDestIndex,order[src].GetName());
//
//  // insert OrderElement into order vector
//  order.insert(order.begin()+dest,1,order[src]);
//
//  // must delete string/element differently depending on
//  // whether element is moved up or down
//
//  // if item is moved down
//  if(nDestIndex > nSrcIndex)
//  {
//    DeleteString(nSrcIndex);
//    order.erase(order.begin()+src);
//    SetCurSel(nDestIndex-1);
//  }
//  else // item is moved up
//  {
//    DeleteString(nSrcIndex+1);
//    order.erase(order.begin()+src+1);
//    SetCurSel(nDestIndex);
//  }
//
//  // set info order
//  iffi.InfoOrder = order;
//
//  // update image file format info
//  m_iff.SetImageFileFormatInfo(iffi);
//
//  // update display of file format at bottom of options dialog
//  m_odlg->SetFormat();
}

void OptionsDlg::OnOptionsDateFormat() 
{
	// TODO: Add your control notification handler code here
  ImageFileFormatInfo iffi = iff.GetImageFileFormatInfo();
	FormatDateDlg dlg;
  int ret;
  dlg.SetFormat(iffi.m_date_format);
  ret = dlg.DoModal();
  if(ret == IDOK)
  {
    iffi.m_date_format = dlg.GetDateFormat();
    iff.SetImageFileFormatInfo(iffi);
    CString ex = GetImageFileFormat();
    m_sEx.SetWindowText(ex);
  }

}
