/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2005 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootData.cpp: implementation of the RootData class.
//
//////////////////////////////////////////////////////////////////////

#include "../stdafx.h"
#include "RootData.h"


namespace OldData
{

  //////////////////////////////////////////////////////////////////////
  // Construction/Destruction
  //////////////////////////////////////////////////////////////////////

  //empty constructor
  RootData::RootData()
  {
  }

  //empty destructor
  RootData::~RootData()
  {
  }

  // -------------------------------------------------------------------

  // set the passed pointers to the dimensions of the current data
  void RootData::GetDimensions (int *ntubes, int *ntransects, int *nwindows, int *nsessions)
  {
     if(m_data.size() > 0)
     {
        *ntubes = m_data.size();
        if(m_data.at(0).size() > 0)
        {
           if(m_data.at(0).at(0).size() > 0)
              *nwindows = m_data.at(0).at(0).size();
           else
              *nwindows = 0;

           *ntransects = m_data.at(0).size();
        }
        else
           *ntransects = 0;

        *nsessions = m_num_sessions;
     }
     else
     {
        *ntubes = 0;
        *ntransects = 0;
        *nwindows = 0;
        *nsessions = 0;
     }
  }

  // -------------------------------------------------------------------

  // set the dimensions of the data structure to the values passed in
  void RootData::SetDimensions (int ntubes, int ntransects, int nwindows, int nsessions)
  {
     m_data.resize(ntubes);

     //loop through every tube
     for(int tube_num = 0; tube_num < ntubes; tube_num++)
     {
        m_data.at(tube_num).resize(ntransects);
      
        //loop through every transect
        for(int transect_num = 0; transect_num < ntransects; transect_num++)
        {
           m_data.at(tube_num).at(transect_num).resize(nwindows);

           //loop through every window
           for(int window_num = 0; window_num < nwindows; window_num++)
           {
              int noffsets = m_data.at(tube_num).at(transect_num).at(window_num).offset_list.size();
              m_data.at(tube_num).at(transect_num).at(window_num).offset_list.resize(nsessions);

              //loop through every offset
              for(int offset_num = noffsets; offset_num < nsessions; offset_num++)
              {
                 CPoint new_offset;
                 new_offset.x = 0;
                 new_offset.y = 0;
                 m_data.at(tube_num).at(transect_num).at(window_num).offset_list.at(offset_num) = new_offset;
              } //end offset for loop
           } //end window for loop
        } //end transect for loop
     } //end tube for loop

     m_num_sessions = nsessions;
  }

  // -------------------------------------------------------------------

  //returns a window given the values passed in
  RootList *RootData::GetRootList(int num_tube, int num_transect, int num_window)
  {
     int tube, trans, win, sess;

     GetDimensions(&tube, &trans, &win, &sess);

     //if the values passed in are out of range, then return NULL
     if((num_tube > tube) || (num_transect > trans) || (num_window > win))
        return NULL;
     //otherwise, return the window
     else
        return &m_data.at(num_tube).at(num_transect).at(num_window);
  }

  // -------------------------------------------------------------------

  //returns a window given the values passed in, but the values cannot be changed
  const RootList *RootData::GetRootList(int num_tube, int num_transect, int num_window) const
  {
     return &m_data.at(num_tube).at(num_transect).at(num_window);
  }

  // -------------------------------------------------------------------

  //clears the data structure
  void RootData::Clear()
  {
     if(m_data.size() > 0)
     {
        if(m_data.at(0).size() > 0)
        {
           if(m_data.at(0).at(0).size() > 0)
              m_data.at(0).at(0).clear();

           m_data.at(0).clear();
        }

     }
     m_data.clear();

  }

  // -------------------------------------------------------------------

  //returns a RootOverTime given the specified index
  RootOverTime& RootList::at(int index)
  {
     return m_root_list.at(index);
  }

  // -------------------------------------------------------------------

  //resizes a rootlist to the specified size
  void RootList::resize(int size)
  {
    m_root_list.resize(size);
  }

  // -------------------------------------------------------------------

  //returns the size of a rootlist
  unsigned int RootList::size() const
  {
    return m_root_list.size();
  }

  // -------------------------------------------------------------------

  //put the given RootOverTime onto the rootlist
  void RootList::push_back(const RootOverTime& root)
  {
    m_root_list.push_back(root);
  }

}// namespace OldData